@@ -3,7 +3,7 @@ package Net::MQTT::Simple;
 # use strict;    # might not be available (e.g. on openwrt)
 # use warnings;  # same.
 
-our $VERSION = '1.16';
+our $VERSION = '1.20';
 
 # Please note that these are not documented and are subject to change:
 our $KEEPALIVE_INTERVAL = 60;
@@ -11,6 +11,7 @@ our $PING_TIMEOUT = 10;
 our $RECONNECT_INTERVAL = 5;
 our $MAX_LENGTH = 2097152;    # 2 MB
 our $READ_BYTES = 16 * 1024;  # 16 kB per IO::Socket::SSL recommendation
+our $PROTOCOL_NAME = "MQIsdp";  # MQIsdp in v3.1, MQTT in v3.1.1
 
 my $global;
 
@@ -24,6 +25,8 @@ BEGIN {
 sub _default_port { 1883 }
 sub _socket_error { "$@" }
 
+sub _client_identifier { "Net::MQTT::Simple[$$]" }
+
 # Carp might not be available either.
 sub _croak {
     die sprintf "%s at %s line %d.\n", "@_", (caller 1)[1, 2];
@@ -33,14 +36,14 @@ sub filter_as_regex {
     my ($filter) = @_;
 
     return "^(?!\\\$)" if $filter eq '#';   # Match everything except /^\$/
-    return "^(?!\\\$)" if $filter eq '/#';  # Match everything except /^\$/
+    return "^/"        if $filter eq '/#';  # Parent (empty topic) is invalid
 
     $filter = quotemeta $filter;
 
-    $filter =~ s{ \z (?<! \\ \/ \\ \# ) }"\\z"x;       # Anchor unless /#$/
-    $filter =~ s{ \\ \/ \\ \#           }""x;
-    $filter =~ s{ \\ \+                 }"[^/]*+"xg;
-    $filter =~ s{ ^ (?= \[ \^ / \] \* ) }"(?!\\\$)"x;  # No /^\$/ if /^\+/
+    $filter =~ s{ \z (?<! \\ \/ \\ \# ) } {\\z}x;       # Anchor unless /#$/
+    $filter =~ s{ \\ \/ \\ \#           } {}x;
+    $filter =~ s{ \\ \+                 } {[^/]*+}xg;
+    $filter =~ s{ ^ (?= \[ \^ / \] \* ) } {(?!\\\$)}x;  # No /^\$/ if /^\+/
 
     return "^$filter";
 }
@@ -135,24 +138,42 @@ sub _send {
 sub _send_connect {
     my ($self) = @_;
 
-    $self->_send("\x10" . _prepend_variable_length(
-        "\0\x06MQIsdp\x03\x02"
-        . pack("n", $KEEPALIVE_INTERVAL)
-        . pack("n/a*", "Net::MQTT::Simple[$$]")
-    ));
+    $self->_send("\x10" . _prepend_variable_length(pack(
+        "x C/a* C C n n/a*",
+        $PROTOCOL_NAME,
+        0x03,
+        0x02,
+        $KEEPALIVE_INTERVAL,
+        $self->_client_identifier
+    )));
 }
 
 sub _send_subscribe {
-    my ($self) = @_;
+    my ($self, @topics) = @_;
 
-    return if not exists $self->{sub};
+    if (not @topics) {
+        @topics = keys %{ $self->{sub} } or return;
+    }
+    return if not @topics;
 
-    # Hardcoded "packet identifier" \0\0 for now. Hello? This is TCP.
-    $self->_send("\x82" . _prepend_variable_length(
-        "\0\0" .
-        join("", map "$_\0",
-            map pack("n/a*", $_), keys %{ $self->{sub} } 
-        )
+    utf8::encode($_) for @topics;
+
+    # Hardcoded "packet identifier" \0\0 for now.
+    $self->_send("\x82" . _prepend_variable_length("\0\0" .
+        pack("(n/a* x)*", @topics)  # x = QoS 0
+    ));
+}
+
+sub _send_unsubscribe {
+    my ($self, @topics) = @_;
+
+    return if not @topics;
+
+    utf8::encode($_) for @topics;
+
+    # Hardcoded "packet identifier" \0\0 for now.
+    $self->_send("\xa2" . _prepend_variable_length("\0\0" .
+        pack("(n/a*)*", @topics)
     ));
 }
 
@@ -214,7 +235,7 @@ sub _incoming_publish {
 
     for my $cb (@{ $self->{callbacks} }) {
         if ($topic =~ /$cb->{regex}/) {
-            $cb->{callback}->($topic, $message);
+            $cb->{callback}->($topic, $message, $packet->{retain});
             return;
         }
     }
@@ -282,6 +303,7 @@ sub subscribe {
     while (my ($topic, $callback) = splice @kv, 0, 2) {
         $self->{sub}->{ $topic } = 1;
         push @{ $self->{callbacks} }, {
+            topic => $topic,
             regex => filter_as_regex($topic),
             callback => $callback,
         };
@@ -290,6 +312,18 @@ sub subscribe {
     $self->_send_subscribe() if $self->{socket};
 }
 
+sub unsubscribe {
+    my ($self, @topics) = @_;
+
+    $self->_send_unsubscribe(@topics);
+
+    my $cb = $self->{callbacks};
+    @$cb = grep $_->{topic} ne $_, @$cb
+        for @topics;
+
+    delete @{ $self->{sub} }{ @topics };
+}
+
 sub tick {
     my ($self, $timeout) = @_;
 
@@ -431,6 +465,12 @@ Subscribes to the given topic(s) and registers the callbacks. Note that only
 the first matching handler will be called for every message, even if filter
 patterns overlap.
 
+=head2 unsubscribe(topic[, topic, ...]) {
+
+Unsubscribes from the given topic(s) and unregisters the corresponding
+callbacks. The given topics must exactly match topics that were previously
+used with the C<subscribe> method.
+
 =head2 run(...)
 
 Enters an infinite loop, which calls C<tick> repeatedly. If any arguments
